import pygame
from pygame.locals import *

from data import get_path

class MapRenderer (object):
	spr_size = (16, 16)
	spr_w, spr_h = spr_size

	def __init__(self, map):
		self.map = map
		self.sprites = {}

	def make_dummy_sprite(self, type):
		surf = pygame.Surface(self.spr_size)
		surf.fill(type.color)
		return surf

	def load_sprites(self):
		for tid, t in self.map.terraininfo.types.items():
			for sid, spr in self.load_spriteset(t):
				n = tid << 4 | sid
				self.sprites[n] = spr
	
	def load_sprite(self, n, t):
		opts = t.options.get(n, 1)
		spr = []
		for i in range(opts):
			name = t.name
			if t.neighbors:
				name += '-%x' % n
			if opts > 1:
				name += '-%x' % i

			path = get_path(name + '.png')

			try:
				surf = pygame.image.load(path).convert()
			except pygame.error, e:
				print 'Warning: missing sprite for', t.name, n, e
				surf = pygame.Surface((self.spr_w, self.spr_h))
				surf.fill(t.color)
			
			spr.append(surf)

		if opts > 1:
			return spr
		else:
			return spr[0]
			
	def load_spriteset(self, t):
		sset = []
		for n in range(16):
			sset.append((n, self.load_sprite(n, t)))

		return sset

	def get_prerender_size(self):
		"""Get the size of prerender surface.
		"""
		return (self.map.w * self.spr_w, self.map.h * self.spr_h)

	def render_cities(self, surf):
		sw = self.spr_w
		sh = self.spr_h
		for y in range(self.map.h):
			for x in range(self.map.w):
				val = self.map.cities[x, y]
				col = val * 7
				surf.set_clip((x * sw, y * sh, sw, sh))
				surf.fill((col, col, col))
		surf.set_clip()

	def prerender(self, surf):
		"""Render entire map to surface.

		FIXME: render sprites, not low-level data.
		"""
		sw = self.spr_w
		sh = self.spr_h
		sprites = self.sprites
		terrain = self.map.sprites
		owner = self.map.flags
		types = self.map.terraininfo.types
		for y in range(self.map.h):
			for x in range(self.map.w):
				t = terrain[x, y]
				spr = sprites[t]
				if type(spr) is list:
					spr = spr[hash((x, y)) % len(spr)]
				surf.blit(spr, (x * sw, y * sh))
				if owner[x, y] == 1 and not types[t >> 4].buildable:
					surf.fill((255, 255, 0), (x * sw + 2, y * sh + 2, 2, 2))

	def update(self, surf):
		"""Render only changed bits.
		"""
		sw = self.spr_w
		sh = self.spr_h
		sprites = self.sprites
		terrain = self.map.sprites
		owner = self.map.flags
		types = self.map.terraininfo.types

		for x, y in self.map.dirty:
			t = terrain[x, y] 
			spr = sprites[t]
			if type(spr) is list:
				spr = spr[hash((x, y)) % len(spr)]
			surf.blit(spr, (x * sw, y * sw))
			if owner[x, y] == 1 and not types[t >> 4].buildable:
				surf.fill((255, 255, 0), (x * sw + 2, y * sh + 2, 2, 2))

		self.map.dirty = []

